package com.hopeland.androidpc.example.uhf;

import java.util.HashMap;

import com.hopeland.androidpc.example.PublicData;
import com.hopeland.androidpc.example.R;
import com.clou.uhf.G3Lib.CLReader;
import com.clou.uhf.G3Lib.RFID_Option;
import com.clou.uhf.G3Lib.ClouInterface.IAsynchronousMessage;
import com.hopeland.ui.BaseActivity;
import android.annotation.SuppressLint;
import android.content.DialogInterface;
import android.hardware.usb.UsbManager;
import android.util.Log;


/**
 * @author RFID_lx Activity 基类
 */
public class UHFBaseActivity extends BaseActivity {

	protected static Boolean _UHFSTATE = false; // 模块是否已经打开
	// static int _PingPong_ReadTime = 10000; // 默认是100:3
	// static int _PingPong_StopTime = 300;
	protected static int _NowAntennaNo = 1; // 读写器天线编号
	protected static int _UpDataTime = 0; // 重复标签上传时间，控制标签上传速度不要太快
	protected static int _Max_Power = 30; // 读写器最大发射功率
	protected static int _Min_Power = 0; // 读写器最小发射功率
	protected static int _ReaderAntennaCount = 1;//读写器天线个数
	protected static int low_power_soc = 10;


	//当前连接的ID
	public static String ConnID = "";
	//当前连接索引
	public static int connectIndex = 0;
	//当前选择读标签的天线号，例如“1&2&4”，当前选择1、2、4天线读标签
	protected static String _NowAntennaNoStr="1";

	//传入帧协议的天线端口值
	public static HashMap<Integer, Integer> hm_Ant_value = new HashMap<Integer, Integer>() {
		{
			//Math.pow(2, 0),目前最多支持24天线
			put(1, 1);
			put(2, 2);
			put(3, 4);
			put(4, 8);
			put(5, 16);
			put(6, 32);
			put(7, 64);
			put(8, 128);
			put(9, 256);
			put(10, 512);
			put(11, 1024);
			put(12, 2048);
			put(13, 4096);
			put(14, 8192);
			put(15, 16384);
			put(16, 32768);
			put(17, 65536);
			put(18, 131072);
			put(19, 262144);
			put(20, 524288);
			put(21, 1048576);
			put(22, 2097152);
			put(23, 4194304);
			put(24, 8388608);
		}
	};


	/**
	 * 模块初始化(串口)
	 *
	 * @param log
	 *            接口回调方法
	 * @return 是否初始化成功
	 */
	public Boolean Rfid_RS232_Init(String connParam,IAsynchronousMessage log) {
		Boolean rt = false;
		try {
			if (_UHFSTATE == false) {
				rt = CLReader.CreateSerialConn(connParam, log);
				if (rt) {
					ConnID = PublicData.serialParam = connParam;
					connectIndex=0;
					_UHFSTATE = true;
				}
				Thread.sleep(500);
			} else {
				rt = true;
			}
		} catch (Exception ex) {
			Log.d("debug", "On the UHF electric abnormal:" + ex.getMessage());
		}
		return rt;
	}


	/**
	 * 模块初始化(Tcp)
	 *
	 * @param log
	 *            接口回调方法
	 * @return 是否初始化成功
	 */
	public Boolean Rfid_Tcp_Init(String connParam,IAsynchronousMessage log) {
		Boolean rt = false;
		try {
			if (_UHFSTATE == false) {
				rt = CLReader.CreateTcpConn(connParam, log);
				if (rt) {
					ConnID = PublicData.tcpParam = connParam;
					connectIndex = 1;
					_UHFSTATE = true;
				}
				Thread.sleep(500);
			} else {
				rt = true;
			}
		} catch (Exception ex) {
			Log.d("debug", "On the UHF electric abnormal:" + ex.getMessage());
		}
		return rt;
	}


	/**
	 * 获取USB设备列表
	 *
	 * @return 设备个数
	 */
	public int  GetUsbDeviceList(){
		//通过Context来获取系统服务
		PublicData.mUsbManager = (UsbManager) getSystemService(this.USB_SERVICE);
		//获取Usb设备列表
		PublicData.sPortList = CLReader.GetUSBList(PublicData.mUsbManager);
		//usb连接参数列表
		PublicData.usbListStr = CLReader.GetUsbDeviceStrList(PublicData.sPortList);

		return PublicData.usbListStr.size();

	}

	/**
	 * 模块初始化(Usb)
	 *
	 * @param log
	 *            接口回调方法
	 * @return 是否初始化成功
	 */
	public Boolean Rfid_Usb_Init(String connParam, IAsynchronousMessage log) {
		Boolean rt = false;
		try {
			if (_UHFSTATE == false) {

				if (!CLReader.GetUsbPermission(this, connParam)) {
					return false;
				}

				rt = CLReader.CreateUsbConn(connParam,log);
				if (rt) {
					ConnID = PublicData.usbParam = connParam;
					connectIndex = 2;
					_UHFSTATE = true;
				}
				Thread.sleep(500);
			} else {
				rt = true;
			}
		} catch (Exception ex) {
			Log.d("debug", "On the UHF electric abnormal:" + ex.getMessage());
		}
		return rt;
	}


	/**
	 * 模块初始化(BT4)
	 *
	 * @param log
	 *            接口回调方法
	 * @return 是否初始化成功
	 */
	public Boolean Rfid_BT4_Init(String connParam, IAsynchronousMessage log) {
		Boolean rt = false;
		try {
			if (_UHFSTATE == false) {
				rt = CLReader.CreateBT4Conn(connParam, log);
				if (rt) {
					ConnID = PublicData.bt4Param = connParam;
					connectIndex = 3;
					_UHFSTATE = true;
				}
				Thread.sleep(500);
			} else {
				rt = true;
			}
		} catch (Exception ex) {
			Log.d("debug", "On the UHF electric abnormal:" + ex.getMessage());
		}
		return rt;
	}



	/**
	 * 模块释放
	 */
	public void UHF_Dispose() {
		if (_UHFSTATE == true) {
			CLReader.CloseConn(ConnID);
			_UHFSTATE = false;
		}
	}

	/**
	 * 获得读写器的读写能力
	 */
	@SuppressLint("UseSparseArrays")
	@SuppressWarnings("serial")
	protected void UHF_GetReaderProperty() {
		String propertyStr = CLReader.GetReaderProperty(ConnID);
		//Log.d("Debug", "Get Reader Property:" + propertyStr);
		String[] propertyArr = propertyStr.split("\\|");
		if (propertyArr.length > 3) {
			try {
				_Min_Power = Integer.parseInt(propertyArr[0]);
				_Max_Power = Integer.parseInt(propertyArr[1]);
				_ReaderAntennaCount = Integer.parseInt(propertyArr[2]);
				_NowAntennaNo = hm_Ant_value.get(_ReaderAntennaCount);
			} catch (Exception ex) {
				Log.d("Debug", "Get Reader Property failure and conversion failed!");
			}
		} else {
			Log.d("Debug", "Get Reader Property failure");
		}
	}

	/**
	 * 设置标签上传参数
	 */
	protected void UHF_SetTagUpdateParam() {
		// 先查询当前的设置是否一致，如果不一致才设置
		String	searchRT  = "";
		try {
			searchRT = RFID_Option.GetTagUpdateParam(ConnID);
			String[] arrRT = searchRT.split("\\|");
			if (arrRT.length >= 2) {
				int nowUpDataTime = Integer.parseInt(arrRT[0]);
				Log.d("Debug", "Check the label to upload time:" + nowUpDataTime);
				if (_UpDataTime != nowUpDataTime) {
					String param="1,"+_UpDataTime;
					CLReader.SetTagUpdateParam(ConnID,param); // 设置标签重复上传时间为20ms
					Log.d("Debug", "Sets the label upload time...");
				} else {
				}
			} else {
				Log.d("Debug", "Query tags while uploading failure...");
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected boolean UHF_CheckReadResult(int retval) {
		if (99 == retval) {
			ShowMsg(getString(R.string.uhf_read_power_over),
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog, int which) {
							UHFBaseActivity.this.finish();
						}
					});
			return false;
		}

		return true;
	}
}
